using System;
using System.Configuration;

namespace Samples.AbstractFactory.DAL
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	public abstract class DatabaseFactory
	{			
		#region protected constructor
		protected DatabaseFactory()
		{						
		}
		#endregion

		#region singleton implementation
		private static volatile DatabaseFactory _instance;
		private static object _lock = new object();

		public static DatabaseFactory GetFactory() 
		{
			if (_instance == null) 
			{
				lock (_lock) 
				{
					if (_instance == null) 
					{
						Configuration configuration = GetConfiguration();
						try 
						{							
							_instance = Activator.CreateInstance(Type.GetType(configuration.Type), true) as DatabaseFactory;
						}
						catch (Exception exc) 
						{
							throw new ApplicationException("Chyba pi vytven instance tovrny.", exc);
						}
					}
				}
			}
			return _instance;
		}		
		#endregion

		#region abstract product getters
		public abstract IOrderDALC GetOrderDALC();
		public abstract IProductDALC GetProductDALC();
		#endregion

		#region helper methods
		private static Configuration GetConfiguration() 
		{
			Configuration configuration = ConfigurationSettings.GetConfig("DAL") as Configuration;
			if (configuration != null)
				return configuration;
			else
				throw new ApplicationException("Konfiguran sekce nenalezena.");
		}
		#endregion		
	}
}
